/* Nico Keyboard for ATtiny2313 8MHz by matsubara 2007-2009.*/
/* version 9beta+ english at101*/
/*by takuya matsubara*/
/*http://nicotak.com*/

//      +--_--+
//rst  1|     |20 vcc
//rx   2|     |19 pb7
//tx   3|     |18 pb6
//x1   4|     |17 pb5
//x2   5|     |16 pb4
//pd2  6|     |15 pb3
//pd3  7|     |14 pb2
//pd4  8|     |13 pb1
//pd5  9|     |12 pb0
//gnd 10|     |11 pd6
//      +-----+

// ctrlL[fncL[ɊU
// ctrl + 1 = F1
// ctrl + 2 = F2
//   :      = :
// ctrl + 9 = F9
// ctrl + 0 = F10
// ctrl + A = Alt
// ctrl + C = ctrl
// ctrl + Z = zenkaku
// ctrl + T = TAB
// ctrl + E = ESC

#include <avr/io.h>
#include <avr/pgmspace.h>


#define MASK_MODE	(1<<2)
#define MASK_TX2	(1<<3)
#define MASK_LED	(1<<4)
#define ID_DAT	(1<<5)
#define IC_CLK	(1<<6)

#define LEDOFF	PORTD&=~MASK_LED
#define LEDON	PORTD|=MASK_LED


#define MODE_SIO 0
#define MODE_PS2 1

#define SWNUM_ALT	0	//XCb`ԍ
#define SWNUM_CTRL	1
#define SWNUM_SHIFT	2
#define SWNUM_CURSOR	60
#define SWNUM_NONE 0xff
							//LN^R[h
#define KB_BS		0x08 	//BACK SPACE
#define KB_ENTER	0x0D 	//ENTER
#define KB_ESC		0x1B 	//ESC
#define KB_RUP		0x0E 	//ROLLUP
#define KB_RDOWN	0x0F 	//ROLLDOWN
#define KB_RIGHT	0x1C 	//J[\Eֈړ 	
#define KB_LEFT		0x1D 	//J[\ֈړ 	
#define KB_UP		0x1E 	//J[\ֈړ 	
#define KB_DOWN		0x1F 	//J[\ֈړ 	

unsigned char swflag[64/8];	//rbg1ƃXCb`ON

/*charactor code*/
PROGMEM unsigned char siotable[] = {
        0,  0,	 /*row=0*/
        0,  0, /*CTRL*/
        0,  0, /*SHIFT*/
      '1','!',
      'q','Q',
      'a','A',
      'z','Z',
        0,  0,
      '3','#',	/*row=1*/
      '2','\"',
      'e','E',
      'w','W',
      'd','D',
      's','S',
      'c','C',
      'x','X',
      '5','%',	/*row=2*/
      '4','$',
      't','T',
      'r','R',
      'g','G',
      'f','F',
      'b','B',
      'v','V',
      '7','\'',	/*row=3*/
      '6','&',
      'u','U',
      'y','Y',
      'j','J',
      'h','H',
      'm','M',
      'n','N',
      '9',')',	/*row=4*/
      '8','(',
      'o','O',
      'i','I',
      'l','L',
      'k','K',
      '.','>',
      ',','<',
      '-','=',	/*row=5*/
      '0',  0,
      '@','`',
      'p','P',
      ':','*',
      ';','+',
      '_','_',
      '/','?',
      '^','~',	/*row=6*/
     '\\','|',
      '[','{',
    KB_BS,  0,
      ']','}',
 KB_ENTER,  0,
      ' ',  0,
        0,  0,
        0,  0,	/*row=7*/
        0,  0,
        0,  0,
        0,  0,
    KB_UP,  0,
  KB_DOWN,  0,
  KB_LEFT,  0,
 KB_RIGHT,  0
};

#define SHFT 0x80

/*Make code*/
PROGMEM unsigned char ps2table[] = {
	0,//0x00
	0,//0x01
	0,//0x02
	0,//0x03
	0,//0x04
	0,//0x05
	0,//0x06
	0,//0x07
	0x66,//0x08 BS
	0x0d,//0x09 tab
	0,//0x0A
	0,//0x0B
	0,//0x0C
	0x5a,//0x0D enter
	0,//0x0E
	0,//0x0F
	0,//0x10
	0,//0x11
	0,//0x12
	0,//0x13
	0,//0x14
	0,//0x15
	0,//0x16
	0,//0x17
	0,//0x18
	0,//0x19
	0,//0x1a
	0x76,	//0x1b ESC
	0x74,	//0x1C RIGHT
	0x6b,	//0x1D LEFT
	0x75,	//0x1E UP
	0x72,	//0x1F DOWN
    0x29,   /* 0x20 space  */
    0x16 | SHFT,   /* 0x21 ! [1]*/
    0x52 | SHFT,   /* 0x22 " [']*/
    0x26 | SHFT,   /* 0x23 # [3]*/
    0x25 | SHFT,   /* 0x24 $ [4]*/
    0x2E | SHFT,   /* 0x25 % [5]*/
    0x3d | SHFT,   /* 0x26 & [7]*/
    0x52,   /* 0x27 ' */
    0x46 | SHFT,   /* 0x28 ( [9]*/
    0x45 | SHFT,   /* 0x29 ) [0]*/
    0x3e | SHFT,   /* 0x2A * [8]*/
    0x55 | SHFT,   /* 0x2B + [=]*/
    0x41,   /* 0x2C , */
    0x4e,   /* 0x2D - */
    0x49,   /* 0x2E . */
    0x4a,   /* 0x2F / */
    0x45,   /* 0x30 0 */
    0x16,   /* 0x31 1 */
    0x1e,   /* 0x32 2 */
    0x26,   /* 0x33 3 */
    0x25,   /* 0x34 4 */
    0x2e,   /* 0x35 5 */
    0x36,   /* 0x36 6 */
    0x3d,   /* 0x37 7 */
    0x3e,   /* 0x38 8 */
    0x46,   /* 0x39 9 */
    0x4c | SHFT,   /* 0x3A : [;]*/
    0x4c,   /* 0x3B ; */
    0x41 | SHFT,   /* 0x3C < [,]*/
    0x55,   /* 0x3D = */
    0x49 | SHFT,   /* 0x3E > [.]*/
    0x4a | SHFT,   /* 0x3F ? [/]*/
    0x1e | SHFT,   /* 0x40 @ [2]*/
    0x1c | SHFT,   /* 0x41 A */
    0x32 | SHFT,   /* 0x42 B */
    0x21 | SHFT,   /* 0x43 C */
    0x23 | SHFT,   /* 0x44 D */
    0x24 | SHFT,   /* 0x45 E */
    0x2b | SHFT,   /* 0x46 F */
    0x34 | SHFT,   /* 0x47 G */
    0x33 | SHFT,   /* 0x48 H */
    0x43 | SHFT,   /* 0x49 I */
    0x3b | SHFT,   /* 0x4A J */
    0x42 | SHFT,   /* 0x4B K */
    0x4b | SHFT,   /* 0x4C L */
    0x3a | SHFT,   /* 0x4D M */
    0x31 | SHFT,   /* 0x4E N */
    0x44 | SHFT,   /* 0x4F O */
    0x4d | SHFT,   /* 0x50 P */
    0x15 | SHFT,   /* 0x51 Q */
    0x2d | SHFT,   /* 0x52 R */
    0x1b | SHFT,   /* 0x53 S */
    0x2c | SHFT,   /* 0x54 T */
    0x3c | SHFT,   /* 0x55 U */
    0x2a | SHFT,   /* 0x56 V */
    0x1d | SHFT,   /* 0x57 W */
    0x22 | SHFT,   /* 0x58 X */
    0x35 | SHFT,   /* 0x59 Y */
    0x1a | SHFT,   /* 0x5A Z */
    0x54,   /* 0x5B [ */
    0x5d,   /* 0x5C \ */
    0x5b,   /* 0x5D ] */
    0x36 | SHFT,   /* 0x5E ^ [6]*/
    0x4e | SHFT,   /* 0x5F _ [-]*/
    0x0e,   /* 0x60 ` */
    0x1c ,   /* 0x61 a */
    0x32 ,   /* 0x62 b */
    0x21 ,   /* 0x63 c */
    0x23 ,   /* 0x64 d */
    0x24 ,   /* 0x65 e */
    0x2b ,   /* 0x66 f */
    0x34 ,   /* 0x67 g */
    0x33 ,   /* 0x68 h */
    0x43 ,   /* 0x69 i */
    0x3b ,   /* 0x6A j */
    0x42 ,   /* 0x6B k */
    0x4b ,   /* 0x6C l */
    0x3a ,   /* 0x6D m */
    0x31 ,   /* 0x6E n */
    0x44 ,   /* 0x6F o */
    0x4d ,   /* 0x70 p */
    0x15 ,   /* 0x71 q */
    0x2d ,   /* 0x72 r */
    0x1b ,   /* 0x73 s */
    0x2c ,   /* 0x74 t */
    0x3c ,   /* 0x75 u */
    0x2a ,   /* 0x76 v */
    0x1d ,   /* 0x77 w */
    0x22 ,   /* 0x78 x */
    0x35 ,   /* 0x79 y */
    0x1a ,   /* 0x7A z */
    0x54 | SHFT,   /* 0x7B { [[]+*/
    0x5d | SHFT,   /* 0x7C | [\]*/
    0x5b | SHFT,   /* 0x7D } []]*/
    0x0e | SHFT    /* 0x7E ~ [`] */
};

// +ctrl
PROGMEM unsigned char fnctable[] = {
    '0',0x09,	//SW41     0 = F10
    '1',0x05,	//SW03     1 = F1
    '2',0x06,	//SW09     2 = F2
    '3',0x04,	//SW08     3 = F3	/*row=1*/
    '4',0x0c,	//SW17     4 = F4
    '5',0x03,	//SW16     5 = F5	/*row=2*/
    '6',0x0B,	//SW25     6 = F6
    '7',0x83,	//SW24     7 = F7	/*row=3*/
    '8',0x0A,	//SW33     8 = F8
    '9',0x01,	//SW32     9 = F9	/*row=4*/
    'a',0x11,	//SW05     A = ALT
    'e',0x76,	//SW10     E = ESC
    'c',0x14,	//SW14     C = CTRL
    't',0x0d,	//SW18     T = TAB
    'z',0x0e,	//SW06     Z = ZENKAKU
};


void ps2_sendbyte(unsigned char dat);
void ps2_recvbyte(void);
void timer_wait(unsigned int cnt);

char mode;


#define BAUDRATE 9600        //{[[g

#ifndef F_CPU
#define   F_CPU   8000000    //CPUNbNg 8MHz
#endif

#define  PRESCALE   1        //vXP[l
#define  PRESCALECR 1        //vXP[ݒl


#define  SIO_PORT PORTD      //ISPp|[g
#define  SIO_PIN  PIND       //ISPp|[g
#define  SIO_DDR  DDRD       //ISPp|[go͐ݒ
#define  SIO_TX   1          //Mrbg

//#define  SIO_TIFR    TIFR1   //^C}1tO
#define  SIO_TIFR    TIFR   //^C}1tO
#define  SIO_MAXCNT  0x10000 //^C}ől

#define TCNT_SIO    (SIO_MAXCNT-((F_CPU/PRESCALE)/BAUDRATE))

#define TCNT_1MS    (SIO_MAXCNT-((F_CPU/PRESCALE)/1000))

#define TCNT_500US  (SIO_MAXCNT-((F_CPU/PRESCALE)/(1000000/500)))
#define TCNT_50US   (SIO_MAXCNT-((F_CPU/PRESCALE)/(1000000/50)))
#define TCNT_10US   (SIO_MAXCNT-((F_CPU/PRESCALE)/(1000000/10)))
#define TCNT_5US    (SIO_MAXCNT-((F_CPU/PRESCALE)/(1000000/5)))

#define TCNT_600HZ    (SIO_MAXCNT-((F_CPU/PRESCALE)/600))	//BEEP 600Hz

//void sio_bitwait(unsigned char dat);
void sio_tx(unsigned char sio_txdat);


//-----------------------------------------------------------------------
// EGCg
void timer_wait(unsigned int cnt)
{
    TCNT1 = cnt;
    SIO_TIFR |= (1 << TOV1);  // TIFR̃rbgNA

    while(!(SIO_TIFR & (1 << TOV1)));
    // TOV1rbg1ɂȂ܂
}

void timer_sio(void)
{
    TCNT1 = TCNT_SIO;
    SIO_TIFR |= (1 << TOV1);  // TIFR̃rbgNA

    while(!(SIO_TIFR & (1 << TOV1)));    // TOV1rbg1ɂȂ܂
}


//-----------------------------------------------------------------------
// 1oCgM
// sio_txdatFMf[^0x00-0xff
void sio_tx(unsigned char sio_txdat)
{
	unsigned char bitmask = 1;

    SIO_PORT |= (1<<SIO_TX);	//0
	timer_sio();    //X^[grbg

    //f[^rbg0-7(ʂ瑗M)
    while(bitmask){    
        if(sio_txdat & bitmask)
    	    SIO_PORT &= ~(1<<SIO_TX);	//1
	    else
        	SIO_PORT |= (1<<SIO_TX);	//0

		timer_sio();
        bitmask <<= 1;
    }
    //Xgbvrbg
    SIO_PORT &= ~(1<<SIO_TX);	//1
	timer_sio();
}

//----------------------

void sio_tx2(unsigned char sio_txdat)
{
	unsigned char bitmask = 1;

    //X^[grbg
	PORTD &= ~MASK_TX2;	//0
	timer_sio();

    //f[^rbg0-7(ʂ瑗M)
    while(bitmask){    
        if(sio_txdat & bitmask)
			PORTD |= MASK_TX2;	//1
	    else
			PORTD &= ~MASK_TX2;	//0

		timer_sio();
        bitmask <<= 1;
    }
	PORTD |= MASK_TX2;	//1
	timer_sio();    //Xgbvrbg
}


//----------------------------------
void wait_msec(int i)
{
	while(i--)
	{
		timer_wait(TCNT_1MS);
	}
}

//600Hz /2 = 300Hz
//void beep(void)
//{
//	int i=60;	// 0.1sec
//
//	while(i--)
//	{
//		PORTD ^= MASK_BEEP;
//		timer_wait(TCNT_600HZ);
//	}
//	PORTD &= ~MASK_BEEP;
//}

//   b11 b10 b09 b08 b07 b06 b05 b04 b03 b02  b01
// --+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+    +---
//   | | | | | | | | | | | | | | | | | | | | |    |
//   +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +-+ +----+
//
// -+   +---+---+---+---+---+---+---+---+---+------+--
//  |sta|d0 |d1 |d2 |d3 |d4 |d5 |d6 |d7 |po |stop
//  +---+---+---+---+---+---+---+---+---+---+
#define  PS2_PORT PORTD      //ISPp|[g
#define  PS2_PIN  PIND      //ISPp|[g
#define  PS2_DDR  DDRD       //ISPp|[go͐ݒ
#define  PS2_DAT   0          //rbg
#define  PS2_CLK   1          //rbg

// PS2RlN^ 1bitM
void ps2_sendclk(void)
{
    timer_wait(TCNT_10US);		//10uSec
	PS2_DDR |= (1<<PS2_CLK);	//output Low
    timer_wait(TCNT_50US);		//50uSec
	PS2_DDR &= ~(1<<PS2_CLK);	//output High(nCCs[_X)
    timer_wait(TCNT_10US);		//10uSec
}

char ps2_recvbit(void)
{
	ps2_sendclk();

	return(PS2_PIN & (1<<PS2_DAT));
}

// PS2RlN^ 1bytesM
void ps2_sendbyte(unsigned char dat)
{
	char pari;
	int i,wdat;

	if(dat==0)return;

	if((PS2_PIN & (1<<PS2_CLK))==0)
		return;	//CLKL̏ꍇA

	while((PS2_PIN & (1<<PS2_DAT))==0){		//DATL̏ꍇA(f[^Փ)
		ps2_recvbyte();
//		return;	
	}
	wdat = (int)dat << 1;	//0rbgڂ̓X^[grbg

	pari=0;
	for(i=0; i<8; i++){
		if((1<<i) & dat){
			pari++;		//rbg1̐JEg
		}
	}
	if((pari & 1)==0){
		//rbg1̐
		wdat |= (1<<9);	//rbg9(peB)=1
	}
	wdat |= (1<<10);	//rbg10(Xgbvrbg)=1

	for(i=0; i<11; i++){	//ʂ11rbgM
		if((1 << i) & wdat){
			PS2_DDR &= ~(1<<PS2_DAT);	//1(HighCs[_X)
		}else{
			PS2_DDR |= (1<<PS2_DAT);	//0
		}
		ps2_sendclk();	//send 1bit clock
		if((PS2_PIN & (1<<PS2_CLK))==0)
			break;	//CLKL̏ꍇA
	}
	PS2_DDR &= ~(1<<PS2_DAT);	//1(HighCs[_X)
	wait_msec(2);	// 2msec wait
					// *@ɂă^C~OႤ\
}

// PS2RlN^ 1bytesM
void ps2_recvbyte(void)
{
	if(mode != MODE_PS2)return;

	unsigned char mask=0x01;
	unsigned char bytedata=0x0;

	if((PS2_PIN & (1<<PS2_CLK))==0)
		return;		//CLK=Low̏ꍇA

	if((PS2_PIN & (1<<PS2_DAT))!=0)
		return;		//DATHigh̏ꍇA(f[^Ȃ)

	ps2_recvbit();	//startbit

	while(mask != 0)
	{	//ʂ8rbgM
		if(ps2_recvbit())
			bytedata |= mask;

		mask <<= 1;
	}
	ps2_recvbit();	//paritybit

	if(ps2_recvbit()){	//Stop bitHigh̏ꍇAM
		PS2_DDR |= (1<<PS2_DAT);	//DAT = Low
		ps2_sendclk();	//send 1bit clock
		PS2_DDR &= ~(1<<PS2_DAT);	//1(HighCs[_X)
	}else{
		return;	//Stop bitLow̏ꍇAt[~OG[
	}
	wait_msec(2);	//2msec
					// *@ɂă^C~OႤ\

	//Mf[^ǂƕԐM
	if(bytedata==0xff){		//ResetR}h
		ps2_sendbyte(0xFA);		//Ack
		//(300-500ms)
		wait_msec(600);	//600msec
		ps2_sendbyte(0xAA);		//BAT
	}else if(bytedata==0xf2){	//IDǂݏoR}h
		ps2_sendbyte(0xFA);		//Ack
		ps2_sendbyte(0xab);		//L[{[hID
		ps2_sendbyte(0x83);		//L[{[hID
//	}else if(bytedata==0xED){	//[hZbgR}h
	}else if(bytedata==0xEE){	//GR[R}h
		ps2_sendbyte(0xEE);		//
	}else{	//Cl[uR}h
/*	}else if(bytedata==0xF4){*/	//Cl[uR}h
		ps2_sendbyte(0xfA);		//Ack
	}
}


// chrnum:LN^[R[h
void ps2_sendcode(unsigned char chrnum)
{
	unsigned char scancode=0;
	unsigned char ezero=0;
	char shiftflag=0;

	if(mode != MODE_PS2)return;
	// PS/2 mode

	if(chrnum==0)return;
	if((chrnum>=0x1C)&&(chrnum<=0x1F)) ezero=0xE0;

	// ctrlƓcheck
	if(swflag[0] & 0x40){
		int i;
		for(i=0; i<sizeof(fnctable); i+=2){
			if(chrnum == pgm_read_byte((PGM_P)fnctable + i)){
				i++;
				scancode = pgm_read_byte((PGM_P)fnctable + i);
				break;
			}
		}
	}else{
		scancode = pgm_read_byte((PGM_P)ps2table + chrnum);
	}
	if(scancode==0)return;
	if(scancode & 0x80){
		shiftflag = 1;
		scancode &= 0x7F;
	}

	if(shiftflag){	//shift key ON
		ps2_sendbyte(0x12);		/* make code*/
	}
	ps2_sendbyte(ezero);		/* 0xE0 code(J[\L[)*/
	ps2_sendbyte(scancode);		/* make code*/
//	timer_wait(TCNT_1MS);		//1mSec
	wait_msec(100);	// msec wait
	ps2_sendbyte(ezero);		/* 0xE0 code(J[\L[)*/
	ps2_sendbyte(0xF0);			/* break code*/
	ps2_sendbyte(scancode);

	if(shiftflag){	//shift key OFF
		wait_msec(50);	// msec wait
		ps2_sendbyte(0xF0);		/* break code*/
		ps2_sendbyte(0x12);
	}
}


int main(void)
{
//  L[s[g̃^C~O
//	#define REPEATMAX 1200	//Key Repeat count
//	#define REPEATCYCLE 250	//Key Repeat cycle
	#define REPEATMAX 250	//Key Repeat count
	#define REPEATCYCLE 40	//Key Repeat cycle

	unsigned char swbak[64/8];	//rbg1ƃXCb`ON
	unsigned char bitmask;
	unsigned char swnow;
	unsigned char swxor;
	char rownum;//s
	char swnum;
	int repeatcnt=0;

	PORTD |= MASK_MODE;	//Pullup
	DDRD &= ~MASK_MODE;
	PORTB = 0xff;		//Pullup
	DDRD |= (MASK_LED | ID_DAT | IC_CLK | MASK_TX2);
	PORTD |= (MASK_LED | ID_DAT | MASK_TX2);	//

	if(PIND & MASK_MODE){	//Mode sw=OFF
		// VA|[g
		SIO_DDR |=  (1<<SIO_TX);
		SIO_PORT &= ~(1<<SIO_TX);
		mode=MODE_SIO;
	}else{				//Mode sw=ON
		PS2_PORT &= ~((1<<PS2_DAT)|(1<<PS2_CLK));	//vAbvȂ/Lowo
		PS2_DDR &= ~((1<<PS2_DAT)|(1<<PS2_CLK));	//1(HighCs[_X)
		mode=MODE_PS2;
	}

    TCCR1A= 0;                // ^C}1 [h 
    TCCR1B= PRESCALECR;       // ^C}1 vXP[ݒ

	//WbNIC / backup
	for(rownum=0; rownum<8; rownum++){
		PORTD |=  IC_CLK;	//(74HC164)CLK
		PORTD &= ~IC_CLK;	//(74HC164)CLK

		swbak[(int)rownum] = 0xFF;
		swflag[(int)rownum] = 0;
	}
	wait_msec(500);	//500msec
	LEDOFF;

	rownum=0;
	swnum = 0;
	while(1)
	{
		ps2_recvbyte();

		//1s`8ŝ1Low
		if(rownum==0){
			PORTD &= ~ID_DAT;	//(74HC164)DAT
		}else{
			PORTD |= ID_DAT;	//(74HC164)DAT
		}
	    timer_wait(TCNT_50US);		//50uSec
		PORTD |=  IC_CLK;	//(74HC164)CLK
		PORTD &= ~IC_CLK;	//(74HC164)CLK
 //   	timer_wait(TCNT_50US);		//50uSec
    	timer_wait(TCNT_500US);		//500uSec
 //		timer_wait(TCNT_1MS);		//1mSec

		swnow = PINB;
		swxor = swbak[(int)rownum] ^ swnow;
		swbak[(int)rownum] = swnow;	//back up sw_data

		swnow &= ~swxor;
		// 	ON-->OFF ܂ OFF-->ONւ̕ωOFFɔ

		bitmask = 0x80;
		while(bitmask){
			if(swflag[(int)rownum] & bitmask){		//sw ON
				if((swnow & bitmask)==0){	//ON-->ON  repeat
					if(swnum > SWNUM_SHIFT){
						repeatcnt++;	//Alt/Shift/CtrlȊO̓s[g
					}
					if(repeatcnt > REPEATMAX){
						swflag[(int)rownum] &= ~bitmask;
					}
				}else{	//ON-->OFF
					LEDOFF;
					swflag[(int)rownum] &= ~bitmask;
					repeatcnt=0;
				}
			}else{	//sw OFF
				if((swnow & bitmask)==0){	//OFF-->ON
					unsigned char chrcode;
					unsigned char offset;

					offset = (swnum<<1);
					if(swflag[0] &(0x80>>SWNUM_SHIFT)){
						LEDON;
						offset++;
					}
					chrcode = pgm_read_byte((PGM_P)siotable + offset);

					if(mode==MODE_SIO){	//SIO(RS232C) mode
						if(chrcode)
							sio_tx(chrcode);
					}else{				// PS/2 mode
						ps2_sendcode(chrcode);
					}
					if(chrcode)
						sio_tx2(chrcode);

					swflag[(int)rownum] |= bitmask;
					if(	repeatcnt > REPEATMAX)
						repeatcnt = (REPEATMAX-REPEATCYCLE);
					else
						repeatcnt=0;
				}else{
					//OFF-->OFFƂȂɂ
				}
			}

			bitmask >>= 1;
			swnum++;
		}

		rownum++;
		if(swnum >= 64){	//8x8=64sw
			rownum=0;
			swnum=0;
		}
	}
	return(0);
}
